@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # This file is part of python-opsi.
-# Copyright (C) 2010-2015 uib GmbH <info@uib.de>
+# Copyright (C) 2010-2016 uib GmbH <info@uib.de>
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -45,6 +45,7 @@ try:
 except ImportError:
 	avahi = None
 
+from contextlib import contextmanager
 from datetime import datetime
 from signal import signal, SIGHUP, SIGINT, SIGTERM
 from ctypes import CDLL
@@ -74,7 +75,7 @@ from monitoring import ResourceOpsiconfdMonitoring
 from session import OpsiconfdSessionHandler
 from omb import MessageBusService, OpsiconfdHTTPFactory, OpsiconfdHTTPChannel
 
-__version__ = "4.0.6.11"
+__version__ = "4.0.7.4"
 
 logger = Logger()
 
@@ -126,7 +127,7 @@ class ZeroconfService(object):
 
 class Opsiconfd(OpsiService):
 	def __init__(self, config):
-		self.config = config
+		self.config = {}
 		self._running = False
 		self._backend = None
 		self._root = None
@@ -143,12 +144,10 @@ class Opsiconfd(OpsiService):
 
 		self.authFailureCount = {}
 
+		self.setConfig(config)
 		self._setOpsiLogging()
 		self._setTwistedLogging()
 
-		if 'startTime' not in self.config:
-			self.config['startTime'] = datetime.now()
-
 		logger.comment("""
 ==================================================================
 =             opsi configuration service starting                =
@@ -159,8 +158,12 @@ class Opsiconfd(OpsiService):
 
 	def setConfig(self, config):
 		logger.notice(u"Got new config")
+		oldStarttime = self.config.get('startTime')
 		self.config = config
 
+		if 'startTime' not in self.config:
+			self.config['startTime'] = oldStarttime or datetime.now()
+
 	def isRunning(self):
 		return self._running
 
@@ -213,6 +216,7 @@ class Opsiconfd(OpsiService):
 		if self._sessionHandler:
 			self._sessionHandler.cleanup()
 		self._createSessionHandler()
+		self._logStatistics()
 		self._createStatistics()
 		self._createSite()
 
@@ -447,6 +451,14 @@ class Opsiconfd(OpsiService):
 		self._messageBusService.start()
 
 	def run(self):
+		@contextmanager
+		def collectStatistics():
+			self._createStatistics()
+			try:
+				yield
+			finally:
+				self._logStatistics()
+
 		self._running = True
 		logger.notice(u"Starting opsiconfd main thread")
 		try:
@@ -456,19 +468,19 @@ class Opsiconfd(OpsiService):
 			self._startListeningSocket()
 			self._createBackendInstance()
 			self._createSessionHandler()
-			self._createStatistics()
-			self._createSite()
-			self._startListening()
-			if self.config['loadbalancing']:
-				logger.debug(u"Loadbalancing is activated, zeroconf-publishing is deactivated")
-			else:
-				self._publish()
+			with collectStatistics():
+				self._createSite()
+				self._startListening()
+				if self.config['loadbalancing']:
+					logger.debug(u"Loadbalancing is activated, zeroconf-publishing is deactivated")
+				else:
+					self._publish()
 
-			if self.config["debug"]:
-				self._startListeningShell()
+				if self.config["debug"]:
+					self._startListeningShell()
 
-			if not reactor.running:
-				reactor.run(installSignalHandlers=1)
+				if not reactor.running:
+					reactor.run(installSignalHandlers=1)
 		except Exception as e:
 			logger.logException(e)
 			self.stop()
@@ -476,6 +488,18 @@ class Opsiconfd(OpsiService):
 		logger.notice(u"Opsiconfd main thread exiting...")
 		self._running = False
 
+	def _logStatistics(self):
+		stats = self._statistics
+		logger.debug("Current system status: {0}".format(stats.getStatistics()))
+		logger.notice("Uptime: {0}".format(str(datetime.now() - self.config['startTime'])))
+
+		logger.notice("Statistics: ")
+		logger.notice("Methodname\tCallcount\tAverage processing duration")
+		callStatistics = stats.getRPCCallCounts()
+		callAverages = stats.getRPCAverageDurations()
+		for key in sorted(callStatistics):
+			logger.notice("{name}\t{count}\t{average}".format(name=key, count=callStatistics[key], average='{0:0.3f}s'.format(callAverages[key])))
+
 
 class OpsiconfdInit(Application):
 	def __init__(self):
