@@ -1,41 +1,35 @@
-#!/usr/bin/python
+#! /usr/bin/env python
 # -*- coding: utf-8 -*-
+
+# opsiconfd is part of the desktop management solution opsi
+# (open pc server integration) http://www.opsi.org
+# Copyright (C) 2010-2015 uib GmbH <info@uib.de>
+
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as
+# published by the Free Software Foundation, either version 3 of the
+# License, or (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+
+# You should have received a copy of the GNU Affero General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
 opsi configuration daemon - workers
 
-opsiconfd is part of the desktop management solution opsi
-(open pc server integration) http://www.opsi.org
-
-Copyright (C) 2010-2013 uib GmbH
-
-http://www.uib.de/
-
-All rights reserved.
-
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU Affero General Public License as
-published by the Free Software Foundation, either version 3 of the
-License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU Affero General Public License for more details.
-
-You should have received a copy of the GNU Affero General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-
-@copyright:  uib GmbH <info@uib.de>
-@author: Jan Schneider <j.schneider@uib.de>
-@author: Niko Wenselowski <n.wenselowski@uib.de>
-@license: GNU Affero General Public License version 3
+:copyright: uib GmbH <info@uib.de>
+:author: Jan Schneider <j.schneider@uib.de>
+:author: Niko Wenselowski <n.wenselowski@uib.de>
+:license: GNU Affero General Public License version 3
 """
 
 import base64
 import os
 import re
 import socket
-import zlib
 
 from twisted.internet import defer, threads
 from twisted.python import failure
@@ -48,6 +42,7 @@ from OPSI.Service.Worker import (WorkerOpsi, WorkerOpsiJsonRpc,
 from OPSI.Types import forceHostId, forceHardwareAddress, OpsiAuthenticationError
 from OPSI.Util import (timestamp, objectToHtml, toJson, randomString,
 	decryptWithPrivateKeyFromPEMFile, ipAddressInNetwork, serialize)
+from OPSI.Util.HTTP import deflateDecode, gzipDecode
 from OPSI.Backend.BackendProcess import OpsiBackendProcess
 from OPSI.Backend.BackendManager import BackendAccessControl, backendManagerFactory
 from OPSI.Logger import Logger, LOG_INFO
@@ -66,30 +61,32 @@ class WorkerOpsiconfd(WorkerOpsi):
 
 	def _setLogFile(self, obj):
 		if self.service.config['machineLogs'] and self.service.config['logFile']:
-			logger.setLogFile( self.service.config['logFile'].replace('%m', self.request.remoteAddr.host), object = obj )
+			logger.setLogFile(self.service.config['logFile'].replace('%m', self.request.remoteAddr.host), object=obj)
 
 	def _linkLogFile(self, result):
 		if self.session.hostname and self.service.config['machineLogs'] and self.service.config['logFile']:
-			logger.linkLogFile( self.service.config['logFile'].replace('%m', self.session.hostname), object = self )
+			logger.linkLogFile(self.service.config['logFile'].replace('%m', self.session.hostname), object=self)
 		return result
 
 	def _errback(self, failure):
 		result = WorkerOpsi._errback(self, failure)
-		if (result.code == responsecode.UNAUTHORIZED) and self.request.remoteAddr.host not in (self.service.config['ipAddress'], '127.0.0.1'):
-			if (self.service.config['maxAuthenticationFailures'] > 0):
+		if result.code == responsecode.UNAUTHORIZED and self.request.remoteAddr.host not in (self.service.config['ipAddress'], '127.0.0.1'):
+			if self.service.config['maxAuthenticationFailures'] > 0:
 				if not self.service.authFailureCount.has_key(self.request.remoteAddr.host):
 					self.service.authFailureCount[self.request.remoteAddr.host] = 0
 				self.service.authFailureCount[self.request.remoteAddr.host] += 1
-				if (self.service.authFailureCount[self.request.remoteAddr.host] > self.service.config['maxAuthenticationFailures']):
+				if self.service.authFailureCount[self.request.remoteAddr.host] > self.service.config['maxAuthenticationFailures']:
 					logger.error(u"%s authentication failures from '%s' in a row, waiting 60 seconds to prevent flooding" \
 							% (self.service.authFailureCount[self.request.remoteAddr.host], self.request.remoteAddr.host))
-					#Will prevent flooding, before block for prevention, delete actual remoteAddr to reset the maxAuthenticationFailure marker
+					# Will prevent flooding, before block for prevention,
+					# delete actual remoteAddr to reset the
+					# maxAuthenticationFailure marker
 					del self.service.authFailureCount[self.request.remoteAddr.host]
 					return self._delayResult(60, result)
 		return result
 
 	def _getAuthorization(self):
-		(user, password) = (u'', u'')
+		user = password = u''
 		logger.debug(u"Trying to get username and password from Authorization header")
 		auth = self.request.headers.getHeader('Authorization')
 		if auth:
@@ -97,20 +94,20 @@ class WorkerOpsiconfd(WorkerOpsi):
 				logger.debug(u"Authorization header found (type: %s)" % auth[0])
 				logger.confidential(u"Auth encoded: %s" % auth[1])
 				authString = None
-				if (auth[0].lower() == 'opsi'):
+				if auth[0].lower() == 'opsi':
 					try:
 						authString = unicode(
 							decryptWithPrivateKeyFromPEMFile(
 								base64.decodestring(auth[1]),
 								self.service.config['sslServerKeyFile']), 'latin-1').strip()
-					except Exception as e:
-						logger.logException(e)
+					except Exception as error:
+						logger.logException(error)
 						raise
 				else:
 					authString = unicode(base64.decodestring(auth[1]), 'latin-1').strip()
 
 				parts = authString.split(':')
-				if (len(parts) > 6):
+				if len(parts) > 6:
 					user = u':'.join(parts[:6])
 					password = u':'.join(parts[6:])
 				else:
@@ -118,8 +115,8 @@ class WorkerOpsiconfd(WorkerOpsi):
 					password = u':'.join(parts[1:])
 				user = user.strip()
 				logger.confidential(u"Client supplied username '%s' and password '%s'" % (user, password))
-			except Exception as e:
-				logger.error(u"Bad Authorization header from '%s': %s" % (self.request.remoteAddr.host, e))
+			except Exception as error:
+				logger.error(u"Bad Authorization header from '%s': %s" % (self.request.remoteAddr.host, error))
 		return (user, password)
 
 	def _getCredentials(self):
@@ -130,18 +127,18 @@ class WorkerOpsiconfd(WorkerOpsi):
 			try:
 				(hostname, aliaslist, ipaddrlist) = socket.gethostbyaddr(self.session.ip)
 				user = forceHostId(hostname)
-			except Exception as e:
-				raise Exception(u"No username given and resolve failed: %s" % e)
+			except Exception as error:
+				raise Exception(u"No username given and resolve failed: %s" % error)
 
-		if (user.count('.') >= 2):
+		if user.count('.') >= 2:
 			self.session.isHost = True
-			if (user.find('_') != -1):
+			if '_' in user:
 				user = forceHostId(user.replace('_', '-'))
 		elif re.search('^([0-9a-f]{2})[:-]?([0-9a-f]{2})[:-]?([0-9a-f]{2})[:-]?([0-9a-f]{2})[:-]?([0-9a-f]{2})[:-]?([0-9a-f]{2})$', user):
 			self.session.isHost = True
 			mac = forceHardwareAddress(user)
 			logger.info(u"Found hardware address '%s' as username, searching host in backend" % mac)
-			hosts = self.service._backend.host_getObjects(hardwareAddress = mac)
+			hosts = self.service._backend.host_getObjects(hardwareAddress=mac)
 			if not hosts:
 				raise Exception(u"Host with hardware address '%s' not found in backend" % mac)
 			user = hosts[0].id
@@ -150,16 +147,17 @@ class WorkerOpsiconfd(WorkerOpsi):
 		if self.session.isHost:
 			hosts = None
 			try:
-				hosts = self.service._backend.host_getObjects(type = 'OpsiClient', id = user)
-			except Exception as e:
-				logger.debug(u"Host not found: %s" % e)
+				hosts = self.service._backend.host_getObjects(type='OpsiClient', id=user)
+			except Exception as error:
+				logger.debug(u"Host not found: %s" % error)
 
 			if hosts:
-				if password and hosts[0].getOneTimePassword() and (password == hosts[0].getOneTimePassword()):
+				if password and hosts[0].getOneTimePassword() and password == hosts[0].getOneTimePassword():
 					logger.info(u"Client '%s' supplied one-time password" % user)
 					password = hosts[0].getOpsiHostKey()
 					hosts[0].oneTimePassword = None
 					self.service._backend.host_createObjects(hosts[0])
+
 		return (user, password)
 
 	def _getSessionId(self):
@@ -169,12 +167,14 @@ class WorkerOpsiconfd(WorkerOpsi):
 			(user, password) = self._getAuthorization()
 			if not password:
 				raise OpsiAuthenticationError(u"Application '%s' on client '%s' did neither supply session id nor password" % (self._getUserAgent(), self.request.remoteAddr.host))
+
 		return sessionId
 
 	def _getSession(self, result):
 		WorkerOpsi._getSession(self, result)
-		if self.session.user and (self.session.user.count('.') >= 2):
+		if self.session.user and self.session.user.count('.') >= 2:
 			self.session.isHost = True
+
 		if self.session.isHost and not self.session.hostname:
 			logger.info(u"Storing hostname '%s' in session" % self.session.user)
 			self.session.hostname = self.session.user
@@ -186,6 +186,7 @@ class WorkerOpsiconfd(WorkerOpsi):
 
 		if self.session.authenticated:
 			return result
+
 		try:
 			(self.session.user, self.session.password) = self._getCredentials()
 
@@ -204,8 +205,8 @@ class WorkerOpsiconfd(WorkerOpsi):
 				addressList = []
 				try:
 					(name, aliasList, addressList) = socket.gethostbyname_ex(self.session.hostname)
-				except Exception as e:
-					logger.warning(u"Failed to resolve hostname '%s': %s" % (self.session.hostname, e))
+				except Exception as error:
+					logger.warning(u"Failed to resolve hostname '%s': %s" % (self.session.hostname, error))
 
 				if self.session.ip not in addressList:
 					# Username (FQDN) of peer does not resolve to peer's ip address
@@ -216,7 +217,7 @@ class WorkerOpsiconfd(WorkerOpsi):
 							(self.session.user, self.session.ip) )
 
 			adminNetwork = False
-			if (len(self.service.config['adminNetworks']) == 1) and (self.service.config['adminNetworks'][0] == u'0.0.0.0/0'):
+			if len(self.service.config['adminNetworks']) == 1 and self.service.config['adminNetworks'][0] == u'0.0.0.0/0':
 				adminNetwork = True
 			else:
 				for networkAddress in self.service.config['adminNetworks']:
@@ -253,11 +254,11 @@ class WorkerOpsiconfd(WorkerOpsi):
 
 			if self.service.authFailureCount.has_key(self.request.remoteAddr.host):
 				del self.service.authFailureCount[self.request.remoteAddr.host]
-		except Exception as e:
-			logger.logException(e, LOG_INFO)
+		except Exception as error:
+			logger.logException(error, LOG_INFO)
 			self._freeSession(result)
 			self.service._getSessionHandler().deleteSession(self.session.uid)
-			raise OpsiAuthenticationError(u"Forbidden: %s" % e)
+			raise OpsiAuthenticationError(u"Forbidden: %s" % error)
 		return result
 
 	def _authorize(self):
@@ -265,20 +266,22 @@ class WorkerOpsiconfd(WorkerOpsi):
 
 	def _getBackend(self, result):
 		if self.session.callInstance and self.session.callInterface:
-			if (len(self.session.postpath) == len(self.request.postpath)):
+			if len(self.session.postpath) == len(self.request.postpath):
 				postpathMatch = True
 				for i in range(len(self.request.postpath)):
-					if (self.request.postpath[i] != self.session.postpath[i]):
+					if self.request.postpath[i] != self.session.postpath[i]:
 						postpathMatch = False
+
 				if postpathMatch:
 					return result
+
 			self.session.interface = None
 			self.session.callInstance.backend_exit()
 
 		self.session.postpath = self.request.postpath
 
 		forceGroups = []
-		if (len(self.service.config['adminNetworks']) == 1) and (self.service.config['adminNetworks'][0] == u'0.0.0.0/0'):
+		if len(self.service.config['adminNetworks']) == 1 and self.service.config['adminNetworks'][0] == u'0.0.0.0/0':
 			forceGroups = None
 		else:
 			for networkAddress in self.service.config['adminNetworks']:
@@ -289,39 +292,43 @@ class WorkerOpsiconfd(WorkerOpsi):
 		def _createBackend():
 			self.session.postpath = self.request.postpath
 			self.session.callInstance = backendManagerFactory(
-					user               = self.session.user,
-					password           = self.session.password,
-					forceGroups        = forceGroups,
-					dispatchConfigFile = self.service.config['dispatchConfigFile'],
-					backendConfigDir   = self.service.config['backendConfigDir'],
-					extensionConfigDir = self.service.config['extensionConfigDir'],
-					aclFile            = self.service.config['aclFile'],
-					depotId            = self.service.config['depotId'],
-					postpath           = self.request.postpath,
-					context            = self.service._backend,
-					messageBusNotifier = self.service.config['messageBus'],
-					startReactor       = False)
+				user=self.session.user,
+				password=self.session.password,
+				forceGroups=forceGroups,
+				dispatchConfigFile=self.service.config['dispatchConfigFile'],
+				backendConfigDir=self.service.config['backendConfigDir'],
+				extensionConfigDir=self.service.config['extensionConfigDir'],
+				aclFile=self.service.config['aclFile'],
+				depotId=self.service.config['depotId'],
+				postpath=self.request.postpath,
+				context=self.service._backend,
+				messageBusNotifier=self.service.config['messageBus'],
+				startReactor=False
+			)
 
 		def _spawnProcess():
 			socket = "/var/run/opsiconfd/worker-%s.socket" % randomString(32)
 
-			process = OpsiBackendProcess(socket = socket, logFile = self.service.config['logFile'].replace('%m', self.request.remoteAddr.host))
+			process = OpsiBackendProcess(
+				socket=socket,
+				logFile=self.service.config['logFile'].replace('%m', self.request.remoteAddr.host)
+			)
 			self.session.callInstance = process
 
 			d = process.start()
 			d.addCallback(lambda x: process.callRemote("setLogging", console=logger.getConsoleLevel(), file=logger.getFileLevel()))
 			d.addCallback(lambda x: process.callRemote("initialize",
-							user               = self.session.user,
-							password           = self.session.password,
-							forceGroups        = forceGroups,
-							dispatchConfigFile = self.service.config['dispatchConfigFile'],
-							backendConfigDir   = self.service.config['backendConfigDir'],
-							extensionConfigDir = self.service.config['extensionConfigDir'],
-							aclFile            = self.service.config['aclFile'],
-							depotId            = self.service.config['depotId'],
-							postpath           = self.request.postpath,
-							messageBusNotifier = self.service.config['messageBus'],
-							startReactor       = False))
+							user=self.session.user,
+							password=self.session.password,
+							forceGroups=forceGroups,
+							dispatchConfigFile=self.service.config['dispatchConfigFile'],
+							backendConfigDir=self.service.config['backendConfigDir'],
+							extensionConfigDir=self.service.config['extensionConfigDir'],
+							aclFile=self.service.config['aclFile'],
+							depotId=self.service.config['depotId'],
+							postpath=self.request.postpath,
+							messageBusNotifier=self.service.config['messageBus'],
+							startReactor=False))
 			return d
 
 		modules = self.service._backend.backend_info()['modules']
@@ -353,13 +360,13 @@ class WorkerOpsiconfd(WorkerOpsi):
 
 			def f():
 				if self.session.isHost:
-					hosts = self.service._backend.host_getObjects(['ipAddress', 'lastSeen'], id = self.session.user)
+					hosts = self.service._backend.host_getObjects(['ipAddress', 'lastSeen'], id=self.session.user)
 					if not hosts:
 						raise Exception(u"Host '%s' not found in backend" % self.session.user)
 					host = hosts[0]
-					if (host.getType() == 'OpsiClient'):
+					if host.getType() == 'OpsiClient':
 						host.setLastSeen(timestamp())
-						if self.service.config['updateIpAddress'] and (host.ipAddress != self.session.ip) and (self.session.ip != '127.0.0.1'):
+						if self.service.config['updateIpAddress'] and host.ipAddress != self.session.ip and self.session.ip != '127.0.0.1':
 							host.setIpAddress(self.session.ip)
 						else:
 							# Value None on update means no change!
@@ -381,34 +388,33 @@ class WorkerOpsiconfd(WorkerOpsi):
 
 	def _processOpsiServiceVerificationKey(self, result):
 		try:
-			for (k, v) in self.request.headers.getAllRawHeaders():
-				if (k.lower() == 'x-opsi-service-verification-key'):
+			for key, value in self.request.headers.getAllRawHeaders():
+				if key.lower() == 'x-opsi-service-verification-key':
 					logger.info(u"Adding header x-opsi-service-verification-key")
 					if not isinstance(result, http.Response):
 						result = http.Response()
 					result.headers.setRawHeaders(
 						'X-opsi-service-verification-key',
-						[ decryptWithPrivateKeyFromPEMFile(base64.decodestring(v[0]), self.service.config['sslServerKeyFile']) ]
+						[decryptWithPrivateKeyFromPEMFile(base64.decodestring(value[0]), self.service.config['sslServerKeyFile'])]
 					)
 					return result
-		except Exception as e:
-			logger.logException(e)
-			logger.error(u"Failed to process opsi service verification key: %s" % e)
+		except Exception as error:
+			logger.logException(error)
+			logger.error(u"Failed to process opsi service verification key: %s" % error)
 		return result
 
 
 class WorkerOpsiconfdJsonRpc(WorkerOpsiconfd, WorkerOpsiJsonRpc, MultiprocessWorkerOpsiJsonRpc):
 	def __init__(self, service, request, resource):
-		WorkerOpsiconfd.__init__(self, service, request, resource, multiProcessing = service.config["multiprocessing"])
+		WorkerOpsiconfd.__init__(self, service, request, resource, multiProcessing=service.config["multiprocessing"])
 		WorkerOpsiJsonRpc.__init__(self, service, request, resource)
 
 		modules = self.service._backend.backend_info()['modules']
-		if self.multiProcessing and \
-		(modules.get('valid', False) and modules.get('high_availability', False)):
-				MultiprocessWorkerOpsiJsonRpc.__init__(self, service, request, resource)
+		if self.multiProcessing and (modules.get('valid', False) and modules.get('high_availability', False)):
+			MultiprocessWorkerOpsiJsonRpc.__init__(self, service, request, resource)
 
 	def _getCallInstance(self, result):
-		d = defer.maybeDeferred(self._getBackend,result)
+		d = defer.maybeDeferred(self._getBackend, result)
 
 		def setInterface():
 			self._callInstance = self.session.callInstance
@@ -449,21 +455,42 @@ class WorkerOpsiconfdJsonRpc(WorkerOpsiconfd, WorkerOpsiJsonRpc, MultiprocessWor
 
 	def _decodeQuery(self, result):
 		try:
-			if (self.request.method == 'POST'):
+			if self.request.method == 'POST':
 				contentType = self.request.headers.getHeader('content-type')
-				contentEncoding = None
 				try:
 					contentEncoding = self.request.headers.getHeader('content-encoding')[0].lower()
-				except:
-					pass
+				except Exception:
+					contentEncoding = None
+
 				logger.debug(u"Content-Type: %s, Content-Encoding: %s" % (contentType, contentEncoding))
-				if (contentEncoding == 'gzip') or (contentType and contentType.mediaType.startswith('gzip')):
-					logger.debug(u"Expecting compressed data from client")
-					self.query = zlib.decompress(self.query)
-			self.query = unicode(self.query, 'utf-8')
-		except (UnicodeError, UnicodeEncodeError) as e:
-			self.service.statistics().addEncodingError('query', self.session.ip, self.session.userAgent, unicode(e))
-			self.query = unicode(self.query, 'utf-8', 'replace')
+				if contentType and contentType.mediaType.startswith('gzip'):
+					# Invalid MIME type.
+					# Probably it is gzip-application/json-rpc and therefore
+					# we need to behave like we did before.
+					logger.debug(u"Expecting compressed data from client (backwards compatible)")
+					self.query = deflateDecode(self.query)
+					self.gzip = True
+				elif contentEncoding == 'gzip':
+					logger.debug(u"Expecting gzip compressed data from client")
+					self.query = gzipDecode(self.query)
+					self.gzip = True
+				elif contentEncoding == 'deflate':
+					logger.debug(u"Expecting deflate compressed data from client")
+					self.query = deflateDecode(self.query)
+					self.gzip = True
+
+			if not isinstance(self.query, unicode):
+				self.query = unicode(self.query, 'utf-8')
+		except (UnicodeError, UnicodeEncodeError) as error:
+			logger.logException(error)
+			self.service.statistics().addEncodingError('query', self.session.ip, self.session.userAgent, unicode(error))
+			if not isinstance(self.query, unicode):
+				self.query = unicode(self.query, 'utf-8', 'replace')
+		except Exception as error:
+			logger.logException(error)
+			logger.warning("Unexpected error during decoding of query: {0}".format(error))
+			raise error
+
 		logger.debug2(u"query: %s" % self.query)
 		return result
 
@@ -485,7 +512,6 @@ class WorkerOpsiconfdJsonRpc(WorkerOpsiconfd, WorkerOpsiJsonRpc, MultiprocessWor
 
 class WorkerOpsiconfdJsonInterface(WorkerOpsiconfdJsonRpc, WorkerOpsiJsonInterface):
 	def __init__(self, service, request, resource):
-
 		WorkerOpsiJsonInterface.__init__(self, service, request, resource)
 		WorkerOpsiconfdJsonRpc.__init__(self, service, request, resource)
 
@@ -494,77 +520,78 @@ class WorkerOpsiconfdJsonInterface(WorkerOpsiconfdJsonRpc, WorkerOpsiJsonInterfa
 
 	def _generateResponse(self, result):
 		logger.info(u"Creating opsiconfd interface page")
-		javascript  = u"var currentParams = new Array();\n"
-		javascript += u"var currentMethod = null;\n"
+
+		javascript = [
+			u"var currentParams = new Array();",
+			u"var currentMethod = null;"
+		]
 		currentMethod = u''
 		if self._rpcs:
 			currentMethod = self._rpcs[0].getMethodName()
-			javascript += u"currentMethod = '%s';\n" % currentMethod
-			for i in range(len(self._rpcs[0].params)):
-				param = self._rpcs[0].params[i]
-				javascript += u"currentParams[%d] = '%s';\n" % (i, toJson(param))
+			javascript.append(u"currentMethod = '%s';" % currentMethod)
+			for (i, param) in enumerate(self._rpcs[0].params):
+				javascript.append(u"currentParams[%d] = '%s';" % (i, toJson(param)))
 
 		currentPath = u'interface'
 		selected = u' selected="selected"'
 		for pp in self.request.postpath:
-			currentPath += u'/%s' % pp
+			currentPath = u'{0}/{1}'.format(currentPath, pp)
 			selected = u''
-		javascript += u"path = '%s';\n" % currentPath
+		javascript.append(u"path = '%s';" % currentPath)
 
-		selectPath = u'<option%s>interface</option>' % selected
+		selectPath = [u'<option%s>interface</option>' % selected]
 		for name in self.service.getBackend().dispatcher_getBackendNames():
 			selected = u''
 			path = u'interface/backend/%s' % name
-			if (path == currentPath):
+			if path == currentPath:
 				selected = u' selected="selected"'
-			selectPath += '<option%s>%s</option>' % (selected, path)
+			selectPath.append('<option%s>%s</option>' % (selected, path))
 
 		for name in os.listdir(self.service.config['extensionConfigDir']):
 			if not os.path.isdir(os.path.join(self.service.config['extensionConfigDir'], name)):
 				continue
 			selected = u''
 			path = u'interface/extend/%s' % name
-			if (path == currentPath):
+			if path == currentPath:
 				selected = u' selected="selected"'
-			selectPath += '<option%s>%s</option>' % (selected, path)
-
-		selectMethod = u''
+			selectPath.append('<option%s>%s</option>' % (selected, path))
 
+		selectMethod = []
 		if self._callInterface:
 			for method in self._callInterface:
-				javascript += u"parameters['%s'] = new Array();\n" % (method['name'])
-				for param in range(len(method['params'])):
-					javascript += u"parameters['%s'][%s]='%s';\n" % (method['name'], param, method['params'][param])
+				methodName = method['name']
+				javascript.append(u"parameters['%s'] = new Array();" % methodName)
+				for (index, param) in enumerate(method['params']):
+					javascript.append(u"parameters['%s'][%s]='%s';" % (methodName, index, param))
 				selected = u''
-				if (method['name'] == currentMethod):
+				if methodName == currentMethod:
 					selected = u' selected="selected"'
-				selectMethod += u'<option%s>%s</option>' % (selected, method['name'])
+				selectMethod.append(u'<option%s>%s</option>' % (selected, methodName))
 
-		resultDiv = u'<div id="result">'
+		def wrapInDiv(obj):
+			return u'<div class="json">{0}</div>'.format(obj)
+
+		resultDiv = [u'<div id="result">']
 		if isinstance(result, failure.Failure):
 			error = u'Unknown error'
 			try:
 				result.raiseException()
-			except Exception as e:
-				error = {'class': e.__class__.__name__, 'message': unicode(e)}
+			except Exception as err:
+				error = {'class': err.__class__.__name__, 'message': unicode(err)}
 				error = toJson({"id": None, "result": None, "error": error})
-			resultDiv += u'<div class="json">'
-			resultDiv += objectToHtml(error)
-			resultDiv += u'</div>'
+			resultDiv.append(wrapInDiv(objectToHtml(error)))
 		else:
 			for rpc in self._rpcs:
-				resultDiv += u'<div class="json">'
-				resultDiv += objectToHtml(serialize(rpc.getResponse()))
-				resultDiv += u'</div>'
-		resultDiv += u'</div>'
+				resultDiv.append(wrapInDiv(objectToHtml(serialize(rpc.getResponse()))))
+		resultDiv.append(u'</div>')
 
 		html = interfacePage % {
-			'path':          currentPath,
-			'title':         u'opsiconfd interface page',
-			'javascript':    javascript,
-			'select_path':   selectPath,
-			'select_method': selectMethod,
-			'result':        resultDiv
+			'path': currentPath,
+			'title': u'opsiconfd interface page',
+			'javascript': u'\n'.join(javascript),
+			'select_path': u''.join(selectPath),
+			'select_method': u''.join(selectMethod),
+			'result': u''.join(resultDiv)
 		}
 
 		if not isinstance(result, http.Response):
@@ -595,9 +622,7 @@ class WorkerOpsiconfdDAV(WorkerOpsiDAV, WorkerOpsiconfd):
 
 		if (not self.resource._authRequired or not self.session.isAdmin) and self.request.method not in ('GET', 'PROPFIND', 'OPTIONS', 'USERINFO', 'HEAD'):
 			logger.critical(u"Method '%s' not allowed (read only)" % self.request.method)
-			return http.Response(
-				code	= responsecode.FORBIDDEN,
-				stream	= "Readonly!" )
+			return http.Response(code=responsecode.FORBIDDEN, stream="Readonly!")
 
 		return self.resource.renderHTTP_super(self.request, self)
 
